package be.cylab.java.roc;

import java.util.Objects;

/**
 * Class to represent element.
 */
public class Point implements Comparable<Point> {

    private double score;
    private double true_alert;

    /**
     * Constructor for Point.
     *
     * @param score      double
     * @param true_alert double
     */
    public Point(final double score, final double true_alert) {
        this.setScore(score);
        this.setTrueAlert(true_alert);
    }

    @Override
    public final int compareTo(final Point point) {
        if (Double.compare(this.score, point.score) > 0) {
            return -1;
        } else if (Double.compare(this.score, point.score) < 0) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * @param o
     * @return
     */

    public final boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Point)) {
            return false;
        }
        Point p = (Point) o;
        return score == p.score;
    }

    /**
     * @return
     */
    public final int hashCode() {
        return Objects.hash(score, true_alert);
    }

    /**
     * Setter for Score.
     *
     * @param score
     */
    public final void setScore(final double score) {
        if (score < 0 || score > 1) {
            throw new IllegalArgumentException("Score must be between 0 and 1");
        }
        this.score = score;
    }

    /**
     * Setter for True Alert.
     *
     * @param true_alert
     */
    public final void setTrueAlert(final double true_alert) {
        if (true_alert != 0 && true_alert != 1) {
            throw  new IllegalArgumentException("True Alert must be 0 or 1");
        }
        this.true_alert = true_alert;
    }

    /**
     * Getter for Score.
     *
     * @return
     */
    public final double getScore() {
        return score;
    }

    /**
     * Getter for True Alert.
     *
     * @return
     */
    public final double getTrueAlert() {
        return true_alert;
    }
}
