package be.cylab.java.roc;

import org.junit.jupiter.api.Test;

import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

class RocTest {

    @Test
    void computeRocPoints() {
        Roc roc = new Roc("src/main/resources/RocSampleData.csv");
        List<CurveCoordinates> rocCoordinatesComputed = roc.computeRocPoints();
        List<CurveCoordinates> coordinates = new ArrayList<>();
        coordinates.add(new CurveCoordinates(0, 0));
        coordinates.add(new CurveCoordinates(0, 0.1));
        coordinates.add(new CurveCoordinates(0, 0.2));
        coordinates.add(new CurveCoordinates(0.1, 0.2));
        coordinates.add(new CurveCoordinates(0.1, 0.3));
        coordinates.add(new CurveCoordinates(0.1, 0.4));
        coordinates.add(new CurveCoordinates(0.1, 0.5));
        coordinates.add(new CurveCoordinates(0.2, 0.5));
        coordinates.add(new CurveCoordinates(0.3, 0.5));
        coordinates.add(new CurveCoordinates(0.3, 0.6));
        coordinates.add(new CurveCoordinates(0.4, 0.6));
        coordinates.add(new CurveCoordinates(0.4, 0.7));
        coordinates.add(new CurveCoordinates(0.5, 0.7));
        coordinates.add(new CurveCoordinates(0.5, 0.8));
        coordinates.add(new CurveCoordinates(0.6, 0.8));
        coordinates.add(new CurveCoordinates(0.7, 0.8));
        coordinates.add(new CurveCoordinates(0.8, 0.8));
        coordinates.add(new CurveCoordinates(0.8, 0.9));
        coordinates.add(new CurveCoordinates(0.9, 0.9));
        coordinates.add(new CurveCoordinates(0.9, 1));
        coordinates.add(new CurveCoordinates(1, 1));
        for (int i = 0; i < coordinates.size(); i++) {
            assertEquals(coordinates.get(i).getXAxis(), rocCoordinatesComputed.get(i).getXAxis());
            assertEquals(coordinates.get(i).getYAxis(), rocCoordinatesComputed.get(i).getYAxis());
        }

    }

    @Test
    void computeAUC() {
        double auc = 0.68;
        Roc roc = new Roc("src/main/resources/RocSampleData.csv");
        assertEquals(auc, roc.computeAUC(), 0.00000001);
    }
    @Test
    void computeAUCWithBigDataset() {
        double[] long_data = new double[200000];
        double[] long_alert = new double[long_data.length];
        for (int i = 0; i < long_data.length; i++) {
            long_data[i] = Math.random();
            double trig = Math.random();
            if (trig < 0.5) {
                long_alert[i] = 0.0;
            } else {
                long_alert[i] = 1.0;
            }
        }
        Roc long_roc = new Roc(long_data, long_alert);
        assertTrue(long_roc.computeAUC() < 1.0 && long_roc.computeAUC() > 0.0);

    }
}