package be.cylab.java.roc;

import com.opencsv.CSVReader;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;

import static org.junit.jupiter.api.Assertions.*;

class StatisticsTest {
    private Statistics stats;

    @Test
    void createStatistics() {
        ArrayList points = new ArrayList<>();
        CSVReader reader;
        try {
            reader = new CSVReader(new InputStreamReader(
                    new FileInputStream("src/main/resources/RocSampleData.csv"), StandardCharsets.UTF_8));
            String[] next_record = null;
            while (true) {
                try {
                    next_record = reader.readNext();
                    if (!(next_record != null)) {
                        break;
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
                Point point = new Point(Double.parseDouble(next_record[0]),
                        Double.parseDouble(next_record[1]));
                points.add(point);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        //Threshold 0.5
        stats = new Statistics(points, 0.5);
        assertEquals(6, stats.getTruePositiveCounter());
        assertEquals(4, stats.getFalsePositiveCounter());
        assertEquals(6, stats.getTrueNegativeCounter());
        assertEquals(4, stats.getFalseNegativeCounter());

        assertEquals(0.6, stats.getTruePositiveRate()); //Sensitivity or Recall
        assertEquals(0.4, stats.getFalsePositiveRate()); //Fall-out
        assertEquals(0.6, stats.getTrueNegativeRate()); //Specificity
        assertEquals(0.4, stats.getFalseNegativeRate());

        assertEquals(0.6, stats.getPrecision());
        assertEquals(0.6, stats.getNegativePredictionValue());
        assertEquals(0.4, stats.getFalseDiscoveryRate());
        assertEquals(0.4, stats.getFalseOmissionRate());
        assertEquals(0.6, stats.getAccuracy());
        assertEquals(0.6, stats.getF1Score());


        //Threshold 0.2
        stats = new Statistics(points, 0.2);
        assertEquals(10, stats.getTruePositiveCounter());
        assertEquals(9, stats.getFalsePositiveCounter());
        assertEquals(1, stats.getTrueNegativeCounter());
        assertEquals(0, stats.getFalseNegativeCounter());

        assertEquals(1, stats.getTruePositiveRate()); //Sensitivity or Recall
        assertEquals(0.9, stats.getFalsePositiveRate()); //Fall-out
        assertEquals(0.1, stats.getTrueNegativeRate()); //Specificity
        assertEquals(0, stats.getFalseNegativeRate());

        assertEquals(0.5263, stats.getPrecision(), 0.0001);
        assertEquals(1, stats.getNegativePredictionValue());
        assertEquals(0.4737, stats.getFalseDiscoveryRate(), 0.0001);
        assertEquals(0, stats.getFalseOmissionRate());
        assertEquals(0.55, stats.getAccuracy());
        assertEquals(0.6897, stats.getF1Score(), 0.0001);

        //Threshold 0.7
        stats = new Statistics(points, 0.58);
        assertEquals(3, stats.getTruePositiveCounter());
        assertEquals(1, stats.getFalsePositiveCounter());
        assertEquals(9, stats.getTrueNegativeCounter());
        assertEquals(7, stats.getFalseNegativeCounter());

        assertEquals(0.3, stats.getTruePositiveRate()); //Sensitivity or Recall
        assertEquals(0.1, stats.getFalsePositiveRate(), 0.0001); //Fall-out
        assertEquals(0.9, stats.getTrueNegativeRate(), 0.0001); //Specificity
        assertEquals(0.7, stats.getFalseNegativeRate(), 0.0001);

        assertEquals(0.75, stats.getPrecision(), 0.0001);
        assertEquals(0.5625, stats.getNegativePredictionValue());
        assertEquals(0.25, stats.getFalseDiscoveryRate(), 0.0001);
        assertEquals(0.4375, stats.getFalseOmissionRate());
        assertEquals(0.6, stats.getAccuracy());
        assertEquals(0.4286, stats.getF1Score(), 0.0001);
    }


}